-----------------------------------------------------------------------------------------------
-- Client Lua Script for WhoWindow
-- Copyright (c) NCsoft. All rights reserved
-----------------------------------------------------------------------------------------------
 
require "Window"
require "ChatSystemLib"
 
-----------------------------------------------------------------------------------------------
-- WhoWindow Module Definition
-----------------------------------------------------------------------------------------------
local WhoWindow = {} 
 
-----------------------------------------------------------------------------------------------
-- Constants
-----------------------------------------------------------------------------------------------
-- e.g. local kiExampleVariableMax = 999

local ktClass =
{
	[GameLib.CodeEnumClass.Medic] 			= Apollo.GetString("ClassMedic"),
	[GameLib.CodeEnumClass.Esper] 			= Apollo.GetString("CRB_Esper"),
	[GameLib.CodeEnumClass.Warrior] 		= Apollo.GetString("ClassWarrior"),
	[GameLib.CodeEnumClass.Stalker] 		= Apollo.GetString("ClassStalker"),
	[GameLib.CodeEnumClass.Engineer] 		= Apollo.GetString("ClassEngineer"),
	[GameLib.CodeEnumClass.Spellslinger] 	= Apollo.GetString("ClassSpellslinger"),
}

local ktClassIcon =
{
	[GameLib.CodeEnumClass.Medic] 			= "Icon_Windows_UI_CRB_Medic",
	[GameLib.CodeEnumClass.Esper] 			= "Icon_Windows_UI_CRB_Esper",
	[GameLib.CodeEnumClass.Warrior] 		= "Icon_Windows_UI_CRB_Warrior",
	[GameLib.CodeEnumClass.Stalker]			= "Icon_Windows_UI_CRB_Stalker",
	[GameLib.CodeEnumClass.Engineer] 		= "Icon_Windows_UI_CRB_Engineer",
	[GameLib.CodeEnumClass.Spellslinger] 	= "Icon_Windows_UI_CRB_Spellslinger",
}

local ktPath =
{
	[PlayerPathLib.PlayerPathType_Soldier] 		= Apollo.GetString("PlayerPathSoldier"),
	[PlayerPathLib.PlayerPathType_Settler] 		= Apollo.GetString("PlayerPathSettler"),
	[PlayerPathLib.PlayerPathType_Scientist] 	= Apollo.GetString("PlayerPathScientist"),
	[PlayerPathLib.PlayerPathType_Explorer] 	= Apollo.GetString("PlayerPathExplorer"),
}

local ktPathIcon =
{
	[PlayerPathLib.PlayerPathType_Soldier] 		= "Icon_Windows_UI_CRB_Soldier",
	[PlayerPathLib.PlayerPathType_Settler] 		= "Icon_Windows_UI_CRB_Colonist",
	[PlayerPathLib.PlayerPathType_Scientist] 	= "Icon_Windows_UI_CRB_Scientist",
	[PlayerPathLib.PlayerPathType_Explorer] 	= "Icon_Windows_UI_CRB_Explorer",
}

local ktSortButtonToFuncMap =
{
	["UserSortBtnName"] = "SortName",
	["UserSortBtnLevel"] = "SortLevel",
	["UserSortBtnClass"] = "SortClass",
	["UserSortBtnPath"] = "SortPath",
	["UserSortBtnZone"] = "SortZone",
}
 
-----------------------------------------------------------------------------------------------
-- Initialization
-----------------------------------------------------------------------------------------------
function WhoWindow:new(o)
    o = o or {}
    setmetatable(o, self)
    self.__index = self 

    -- initialize variables here
	self.lastResponse = nil
	self.lastSort = {}

    return o
end

function WhoWindow:Init()
    Apollo.RegisterAddon(self)
end
 

-----------------------------------------------------------------------------------------------
-- WhoWindow OnLoad
-----------------------------------------------------------------------------------------------
function WhoWindow:OnLoad()
    -- Register handlers for events, slash commands and timer, etc.
    -- e.g. Apollo.RegisterEventHandler("KeyDown", "OnKeyDown", self)
	
	self.xmlDoc = XmlDoc.CreateFromFile("WhoWindow.xml")
    self.xmlDoc:RegisterCallback("OnDocLoaded", self)
	
	
	Apollo.RegisterEventHandler("WhoResponse", "OnWhoResponse", self)

end

-----------------------------------------------------------------------------------------------
-- WhoWindow OnDocLoaded
-----------------------------------------------------------------------------------------------


function WhoWindow:OnDocLoaded()
	if self.xmlDoc ~= nil and self.xmlDoc:IsLoaded() then
		self.wndMain = Apollo.LoadForm(self.xmlDoc, "WhoForm", nil, self)
		if self.wndMain == nil then
			Apollo.AddAddonErrorText(self, "Could not load the who window.")
			return
		end
		self.wndListContainer = self.wndMain:FindChild("ListContainer")
		if self.wndListContainer == nil then
			Apollo.AddAddonErrorText(self, "Could not load the who window contents.")
			return
		end
		
		self.wndFilter = self.wndMain:FindChild("FilterBackground"):FindChild("WhoFilter")
		
		self.wndMain:Show(false, true)
	end
end

-----------------------------------------------------------------------------------------------
-- WhoWindow Functions
-----------------------------------------------------------------------------------------------

function WhoWindow:OnWhoResponse(arResponse, eWhoResult)
	if eWhoResult == GameLib.CodeEnumWhoResult.OK or eWhoResult == GameLib.CodeEnumWhoResult.Partial then
		if arResponse == nil or #arResponse == 0 then
			ChatSystemLib.PostOnChannel(ChatSystemLib.ChatChannel_System, Apollo.GetString("Who_NoResults"))
			return
		end

		self.lastResponse = arResponse;
		self.wndMain:Show(true)
		
		self:DrawList()
				
		--[[for _, tWhoExt in ipairs(arResponse) do
			-- each line in arResponse has strName, nLevel, eRaceId, eClassId, ePlayerPathType, nWorldZone, strRace, strClass, strZone, strPath, the last 4 can be nil
			local strLine = String_GetWeaselString(Apollo.GetString("Who_Listing"), tWhoExt.strName, tWhoExt.nLevel, tWhoExt.strRace or "", tWhoExt.strClass or "", tWhoExt.strPath or "", tWhoExt.strZone or "")
		
			ChatSystemLib.PostOnChannel(ChatSystemLib.ChatChannel_System, strLine, "")
		end]]--
	elseif eWhoResult == GameLib.CodeEnumWhoResult.UnderCooldown then
		ChatSystemLib.PostOnChannel(ChatSystemLib.ChatChannel_System, Apollo.GetString("Who_UnderCooldown"), "")
	end
end

-----------------------------------------------------------------------------------------------
-- WhoWindow Drawing
-----------------------------------------------------------------------------------------------

function WhoWindow:DrawList()	
	self.wndListContainer:DestroyChildren()
	self.wndListContainer:RecalculateContentExtents()
	
	for _, tUser in ipairs(self.lastResponse) do
		self:AddOrUpdateUserWindow(tUser)
	end	
	
	self.wndListContainer:ArrangeChildrenVert(0)
end

function WhoWindow:AddOrUpdateUserWindow(tUser)
	if not self.wndMain or not self.wndMain:IsValid() then -- May actually not be visible yet
		return
	end
		
	--Print("Adding: " .. tUser.strName)
	
	local wndNewUserEntry = self:GetExistingUserWindowByName(tUser.strName)
	if wndNewUserEntry == nil then
		wndNewUserEntry = self:FactoryProduce(self.wndListContainer, tUser.strName)
		
		if wndNewUserEntry == nil then
			return
		end
	end
	
	if wndNewUserEntry:FindChild("Name") then
		wndNewUserEntry:FindChild("Name"):SetText(tUser.strName)
	end
	if wndNewUserEntry:FindChild("Level") then
		wndNewUserEntry:FindChild("Level"):SetText(tUser.nLevel)
	end
	if wndNewUserEntry:FindChild("Zone") then
		wndNewUserEntry:FindChild("Zone"):SetText(tUser.strZone)
	end
	self:FormatPathAndClass(wndNewUserEntry, tUser.eClassId, tUser.ePlayerPathType)
	
	
	--update
end

function WhoWindow:FactoryProduce(wndParent, strData)
	local wndNew = wndParent:FindChildByUserData(strData)
	if not wndNew then
		wndNew = Apollo.LoadForm(self.xmlDoc, "UserForm", wndParent, self)
		wndNew:SetData(strData)
	end
	
	return wndNew
end

function WhoWindow:FormatPathAndClass(wndUserEntry, nClassId, nPathId)
	local wndClass = wndUserEntry:FindChild("Class")
	if wndClass ~= nil then
		if ktClassIcon[nClassId] ~= nil then
			wndClass:SetData(nClassId)
			wndClass:SetSprite(ktClassIcon[nClassId])
		end
		if ktClass[nClassId] ~= nil then
			wndClass:SetTooltip(ktClass[nClassId])
		end
	end

	local wndPath = wndUserEntry:FindChild("Path")
	if wndPath ~= nil then
		if ktPathIcon[nPathId] ~= nil then
			wndPath:SetData(nPathId)
			wndPath:SetSprite(ktPathIcon[nPathId])
		end
		if ktPath[nPathId] ~= nil then
			wndPath:SetTooltip(ktPath[nPathId])
		end
	end
end

-----------------------------------------------------------------------------------------------
-- WhoWindow Helper Funcs
-----------------------------------------------------------------------------------------------


function WhoWindow:GetExistingUserWindowByName(strName)
	if not self.wndMain or not self.wndMain:IsValid() then
		return
	end

	for key, wndUserEntry in pairs(self.wndListContainer:GetChildren()) do
		if wndUserEntry:GetData() == strName then
			return wndUserEntry
		end
	end
	
	return nil
end

---------------------------------------------------------------------------------------------------
-- UsersForm Functions
---------------------------------------------------------------------------------------------------

function WhoWindow:OnUserBtn( wndHandler, wndControl, eMouseButton )
end

function WhoWindow:OnUserBtnUncheck( wndHandler, wndControl, eMouseButton )
end

function WhoWindow:OnContextMenuOnlyBtn( wndHandler, wndControl, eMouseButton )
	local strName = wndControl:GetParent():GetData()
	Event_FireGenericEvent("GenericEvent_NewContextMenuPlayerDetailed", wndHandler, strName)
end

---------------------------------------------------------------------------------------------------
-- WhoForm Functions
---------------------------------------------------------------------------------------------------

function WhoWindow:OnRefresh( wndHandler, wndControl, eMouseButton )
	local com = "/who"
	local filter = self.wndFilter:GetText()
	if(filter ~= nil and filter ~= "") then
		com = com .. " " .. filter
	end
	ChatSystemLib.Command(com)
end

function WhoWindow:OnCancel( wndHandler, wndControl, eMouseButton )
	self.wndMain:Show(false);
end

function WhoWindow:OnSortToggle( wndHandler, wndControl, eMouseButton )
	local wndName = wndControl:GetName()
	local bDesc = true
	local tLastSort = self.lastSort
	local strLastSort = tLastSort[2]
		
	if strLastSort == wndName then
		bDesc = true
		self.lastSort = {wndName,wndName.."_desc"}
	elseif strLastSort == wndName.."_desc" then
		self.wndListContainer:ArrangeChildrenVert(0, function(wndLeft, wndRight)
			return self:SortDefault(wndLeft, wndRight)
		end)
		self.lastSort = {wndName, "Default"}
		wndControl:SetCheck(false)
		return
	else
		bDesc = false
		self.lastSort = {wndName, wndName}
	end
	
	self.wndListContainer:ArrangeChildrenVert(0, function(wndLeft, wndRight)
		if ktSortButtonToFuncMap[wndName] ~= nil then
			return self[ktSortButtonToFuncMap[wndName]](self, bDesc, wndLeft, wndRight)
		end
	end)
	
end

function WhoWindow:PerformLastSort()
	local tLastSort = self.lastSort
	local strLastSort = tLastSort[2]
	local wndName = tLastSort[1]
	local bDesc = true
	
	if strLastSort == wndName.."_desc" then
		bDesc = true
	elseif strLastSort == "Default" then
		self.wndListContainer:ArrangeChildrenVert(0, function(wndLeft, wndRight)
			return self:SortDefault(wndLeft, wndRight)
		end)
	else
		bDesc = false
	end
	
	if wndName ~= "" then
		self.arHeaders[eCurrentTab]:FindChild(wndName):SetCheck(true)
	end
	
	self.wndListContainer:ArrangeChildrenVert(0, function(wndLeft, wndRight)
		if ktSortButtonToFuncMap[wndName] ~= nil then
			return self[ktSortButtonToFuncMap[wndName]](self, bDesc, wndLeft, wndRight)
		end
	end)
end

-----------------------------------------------------------------------------------------------
-- WhoWindow Sort
-----------------------------------------------------------------------------------------------

function WhoWindow:SortDefault(wndLeft, wndRight)
	return self:SortName(false, wndLeft, wndRight)
end

function WhoWindow:SortName(bDesc, wndLeft, wndRight)
	local strLeft = wndLeft:FindChild("Name"):GetText()
	local strRight = wndRight:FindChild("Name"):GetText()
	
	if bDesc then
		return strLeft > strRight
	end
	
	return strLeft < strRight
end

function WhoWindow:SortLevel(bDesc, wndLeft, wndRight)
	local levelLeft = tonumber(wndLeft:FindChild("Level"):GetText())
	local levelRight = tonumber(wndRight:FindChild("Level"):GetText())
	
	if bDesc then
		return levelLeft > levelRight
	end
	
	return levelLeft < levelRight
end

function WhoWindow:SortClass(bDesc, wndLeft, wndRight)
	local classLeft = wndLeft:FindChild("Class"):GetData()
	local classRight = wndRight:FindChild("Class"):GetData()
	
	if bDesc then
		return classLeft > classRight
	end
	
	return classLeft < classRight
end

function WhoWindow:SortPath(bDesc, wndLeft, wndRight)
	local pathLeft = wndLeft:FindChild("Path"):GetData()
	local pathRight = wndRight:FindChild("Path"):GetData()
	
	if bDesc then
		return pathLeft > pathRight 
	end
	
	return pathLeft < pathRight 
end

function WhoWindow:SortZone(bDesc, wndLeft, wndRight)
	local strLeft = wndLeft:FindChild("Zone"):GetText()
	local strRight = wndRight:FindChild("Zone"):GetText()
	
	if bDesc then
		return strLeft > strRight
	end
	
	return strLeft < strRight

end

-----------------------------------------------------------------------------------------------
-- WhoWindow Instance
-----------------------------------------------------------------------------------------------
local WhoWindowInst = WhoWindow:new()
WhoWindowInst:Init()
